require 'fast_helper'
require 'background_jobs/strategies/sidekiq/job_attributes_adapter'

module BackgroundJobs
  module Sidekiq
    describe JobAttributesAdapter do
      context 'date' do
        let(:attributes) { [Date.parse('1st Jan 2011')] }
        let(:attributes_encoded) { ['date:2011-01-01'] }

        it 'does decode to string with "date:"" prefix' do
          JobAttributesAdapter.new(attributes_encoded).decode.should eq attributes
        end

        it 'does encode to string with "date:"" prefix' do
          JobAttributesAdapter.new(attributes).encode.should eq attributes_encoded
        end
      end

      context 'date range' do
        let(:attributes) { [Date.parse('1st Jan 2011')..Date.parse('1st Feb 2011')] }
        let(:attributes_encoded) { ['range:[date:2011-01-01,date:2011-02-01]'] }

        it 'does decode to string with "range:" prefix and encode its boundary dates' do
          JobAttributesAdapter.new(attributes_encoded).decode.should eq attributes
        end

        it 'does encode to string with "range:" prefix and encode its boundary dates' do
          JobAttributesAdapter.new(attributes).encode.should eq attributes_encoded
        end
      end

      context 'integers range' do
        let(:attributes) { [1..10] }
        let(:attributes_encoded) { ['range:[1,10]'] }

        it 'does decode to string with prefix range:' do
          JobAttributesAdapter.new(attributes_encoded).decode.should eq attributes
        end

        it 'does encode to string with prefix range:' do
          JobAttributesAdapter.new(attributes).encode.should eq attributes_encoded
        end
      end

      context 'integer' do
        let(:attributes) { [12345] }
        let(:attributes_encoded) { [12345] }

        it 'is left decodehout any tranformations' do
          JobAttributesAdapter.new(attributes_encoded).decode.should eq attributes
        end

        it 'does encode to string with prefix range:' do
          JobAttributesAdapter.new(attributes).encode.should eq attributes_encoded
        end
      end

      context 'hash' do
        let(:attributes) { [{:foo => 'bar'}] }
        let(:attributes_encoded) { [{'foo' => 'bar'}] }

        it 'is left decodehout any tranformations' do
          JobAttributesAdapter.new(attributes_encoded).decode.should eq attributes
        end

        it 'does encode to string with prefix range:' do
          JobAttributesAdapter.new(attributes).encode.should eq attributes_encoded
        end
      end

      context 'integer, hash and range' do
        let(:attributes) { [12345, {:foo => 'bar'}, 10..1000] }
        let(:attributes_encoded) { [12345, {'foo'=>"bar"}, "range:[10,1000]"] }

        it 'does decode all together appropriately to their type' do
          JobAttributesAdapter.new(attributes_encoded).decode.should eq attributes
        end

        it 'does encode to string with prefix range:' do
          JobAttributesAdapter.new(attributes).encode.should eq attributes_encoded
        end
      end

      context 'unknown group id bug' do
        let(:attributes) { 
          [
            "624ea403ad028e03045ba34c923bce0b2a812099bcd815bb1ddf2d92d8aaa560", 
            {
              :group_id => 627, 
              :date_range => [Date.parse("2016-12-31")], 
              :week_schedule => {
                :monday => 0, 
                :tuesday => 1, 
                :wednesday => 2, 
                :thursday => 3, 
                :friday => 2, 
                :saturday => 0, 
                :sunday => 0
              }
            }
          ]
        }

        let(:attributes_encoded) { 
          [
            "624ea403ad028e03045ba34c923bce0b2a812099bcd815bb1ddf2d92d8aaa560", 
            {
              "group_id"=>627, 
              "date_range"=>["date:2016-12-31"], 
              "week_schedule"=>{"monday"=>0, "tuesday"=>1, "wednesday"=>2, "thursday"=>3, "friday"=>2, "saturday"=>0, "sunday"=>0}
            }
          ]
        }

        it 'does encode well' do
          JobAttributesAdapter.new(attributes).encode.should eq attributes_encoded
        end

        it 'does decode well' do
          decoded = JobAttributesAdapter.new(attributes_encoded).decode

          decoded.should eq attributes
          decoded.last[:group_id].should == 627
        end
      end
    end
  end
end